import argparse
import os
import time
import numpy as np
from tqdm import tqdm
from openpyxl import Workbook
from openpyxl.styles import Font

import sys

project_path = os.getcwd()
sys.path.insert(0, project_path + '/utils')

from file_utils import FileUtils


def files_in_dir(dir_list: list, file_str: str):
    file_idx = []

    for idx, file in enumerate(dir_list):
        if file_str in file:
            file_idx.append(idx)

    return file_idx


parser = argparse.ArgumentParser(
    description='Script for parsing CSV files to XLSX'
)

parser.add_argument('-out_file_name', type=str)
parser.add_argument('-json_conf', default='csv_format/default.json', type=str)
parser.add_argument('-csv_paths', nargs='+', type=str)
parser.add_argument('-show_columns', nargs='+', type=str)

args = parser.parse_args()

dir_files = FileUtils.listSubFiles('./csv_format')

# Checks json files in dir and collects their paths into a list
if args.json_conf is not None and os.path.exists(args.json_conf):
    # Checks if file exists
    if not os.path.exists(args.json_conf):
        raise ValueError('Invalid json path: ' + f'{args.json_conf}')

    json_path = args.json_conf
else:
    # Checks if there are any json files in dir
    if not any('.json' in file for file in dir_files):
        raise ValueError('There is no configuration json files in the directory')

    # Collects all json files in dir and chooses first json file path
    json_idx = files_in_dir(dir_files, '.json')[0]
    json_path = dir_files[json_idx]

# Checks CSV files in dir and collects their paths into a list
if args.csv_paths is not None:
    # Checks if file exists
    for path in args.csv_paths:
        if not os.path.exists(path):
            raise ValueError('Invalid CSV path: ' + f'{path}')

    csv_paths = args.csv_paths
else:
    # Checks if there are any CSV files in dir
    if not any('.csv' in file for file in dir_files):
        raise ValueError('There is no configuration json files in the directory')

    # Collects csv file paths into a list
    csv_idx = files_in_dir(dir_files, '.csv')
    csv_paths = [dir_files[idx] for idx in csv_idx]

config = args.__dict__
json_config = FileUtils.readJSON(json_path)

# If any args values were None then data from the json file is used
for key, val in json_config.items():
    if config[key] is None:
        config[key] = val

for csv_path in csv_paths:
    wb = Workbook()
    ws = wb.worksheets[0]

    ws = FileUtils.parseCsvToXlsx(csv_path, ws)

    # Hides all columns that are not in the provided params
    for cell in ws[1]:
        if cell.value not in config['show_columns']:
            ws.column_dimensions[cell.column_letter].hidden = True

    for row in ws:
        for cell in row:
            # Sets styling to each cell
            cell.font = Font(size=10.0, name='Liberation Sans')

            cell.value = float(cell.value) if cell.value.replace('.', '', 1).isdigit() else cell.value

    # Sets output file name and path
    path_cut = csv_path.split('/')
    if config['out_file_name'] == "default":
        out_file_name = "checked_" + path_cut[-1][:-3] + "xlsx"
    else:
        out_file_name = args.out_file_name + ".xlsx"
    path_cut[-1] = out_file_name

    wb.save('/'.join(path_cut))
